<?php
/*
 * Plugin Name: Fsfpay Crypto Payment Gateway
 * Plugin URI: https://fsfpay.com/
 * Description: You can accept payments in cryptocurrencies with using this payment plugin.
 * Author: Fsfpay
 * Author URI: https://fsfpay.com/
 * Version: 1.0.0
 * Text Domain: fsfpay-payment
 * Domain Path: /languages
 *
 */

add_filter('woocommerce_payment_gateways', 'fsfpay_payment_add_gateway_class');
function fsfpay_payment_add_gateway_class($gateways)
{
	$gateways[] = 'WC_FsfpayPayment_Gateway';
	return $gateways;
}
add_action('init', 'fsfpay_init');
function fsfpay_init()
{
	load_plugin_textdomain('fsfpay-payment', false, dirname(plugin_basename(__FILE__)) . '/languages/');
}
add_action('plugins_loaded', 'fsfpay_payment_init_gateway_class');

function fsfpay_payment_init_gateway_class()
{
	if (!class_exists('WC_Payment_Gateway') || class_exists('WC_FsfpayPayment_Gateway')) {
		return;
	}
	class WC_FsfpayPayment_Gateway extends WC_Payment_Gateway
	{
		public string $base_url = 'https://panel.fsfpay.com/';

		public function __construct()
		{
			$this->id = 'fsfpay_payment';
			$this->icon = '';
			$this->has_fields = true;
			$this->callback_url = get_site_url(null, '?wc-api=' . $this->id);
			$this->method_title = __('Fsfpay Cryptocurrency Payment', 'fsfpay-payment');
			$this->method_description = __('You can accept payments in cryptocurrencies with using this payment plugin.', 'fsfpay-payment');
			$this->supports = array('products');
			$this->init_form_fields();
			$this->init_settings();

			$this->title = $this->get_option('title');
			$this->description = $this->get_option('description');
			$this->enabled = $this->get_option('enabled');
			$this->api_username = $this->get_option('api_username');
			$this->api_key = $this->get_option('api_key');
			$this->secret_key = $this->get_option('secret_key');
			$this->order_status = $this->get_option('order_status');
			$this->test_mode = $this->get_option('test_mode');

			add_action('woocommerce_update_options_payment_gateways_' . $this->id, array($this, 'process_admin_options'));
			add_action('woocommerce_receipt_' . $this->id, array($this, 'receipt_page'));
			add_action('woocommerce_api_' . $this->id, array($this, 'callback_handler'));
			add_action('wp_enqueue_scripts', array($this, 'payment_scripts'));
		}

		function fsfpay_request($path, $req = [])
		{
			$data = [
				'apiKey' => $this->api_key,
				'apiUsername' => $this->api_username,
				'requestHash' => hash('sha256', $this->api_key . $this->api_username . $this->secret_key),
				'language' => explode('_', get_locale())[0]
			];
			$data = array_merge($data, $req);
			$ch = curl_init();
			curl_setopt_array($ch, [
				CURLOPT_URL => $this->base_url . 'api/' . $path,
				CURLOPT_HTTPHEADER => [
					'Content-Type: application/x-www-form-urlencoded'
				],
				CURLOPT_POST => true,
				CURLOPT_RETURNTRANSFER => true,
				CURLOPT_POSTFIELDS => http_build_query($data)
			]);
			$response = curl_exec($ch);
			$response = json_decode($response, true);
			curl_close($ch);
			return $response;
		}

		public function init_form_fields()
		{
			$this->form_fields = array(
				'enabled' => array(
					'title'       => __('Enable/Disable', 'fsfpay-payment'),
					'label'       => __('Activate the payment method', 'fsfpay-payment'),
					'type'        => 'checkbox',
					'description' => '',
					'default'     => 'no'
				),
				'title' => array(
					'title'       => __('Title'),
					'type'        => 'text',
					'description' => __('This controls the title which the user sees during checkout.', 'fsfpay-payment'),
					'default'     => __('Pay with Cryptocurrency', 'fsfpay-payment'),
					'desc_tip'    => true
				),
				'description' => array(
					'title'       => __('Description'),
					'type'        => 'textarea',
					'description' => __('This controls the description which the user sees during checkout.', 'fsfpay-payment'),
					'default'     => __('You can easily pay with using cryptocurrency.', 'fsfpay-payment'),
				),
				'api_username' => array(
					'title'       => __('API Username', 'fsfpay-payment'),
					'description' => sprintf(__('You can find this information on the <a href="%s" target="_blank">Merchant Details</a> page.', 'fsfpay-payment'), $this->base_url . 'profile/merchant'),
					'type'        => 'text'
				),
				'api_key' => array(
					'title'       => __('API Key', 'fsfpay-payment'),
					'description' => sprintf(__('You can find this information on the <a href="%s" target="_blank">Merchant Details</a> page.', 'fsfpay-payment'), $this->base_url . 'profile/merchant'),
					'type'        => 'text'
				),
				'secret_key' => array(
					'title'       => __('Secret Key', 'fsfpay-payment'),
					'description' => sprintf(__('You can find this information on the <a href="%s" target="_blank">Merchant Details</a> page.', 'fsfpay-payment'), $this->base_url . 'profile/merchant'),
					'type'        => 'password'
				),
				'order_status' => array(
					'title'       => __('Order Completed Status', 'fsfpay-payment'),
					'description' => __('Order status will be changed to this status after payment confirmation.', 'fsfpay-payment'),
					'type'        => 'select',
					'options'     => array(
						'processing' => __('Processing', 'fsfpay-payment'),
						'completed' => __('Completed', 'fsfpay-payment')
					),
					'default' => 'processing'
				),
				'test_mode' => array(
					'title'       => __('Test Mode', 'fsfpay-payment'),
					'label'       => __('Enable the test mode', 'fsfpay-payment'),
					'type'        => 'checkbox',
					'description' => '',
					'default'     => 'no'
				)
			);
		}

		public function payment_fields()
		{
			echo $this->description;
		}
		public function receipt_page($order_id)
		{
			$order = new WC_Order($order_id);
			$order_data = $order->get_data();
			$currency = strtolower($order_data['currency']);

			if (!isset($_GET['coin']) || empty($_GET['coin'])) {
				$response = $this->fsfpay_request('cryptocurrencies');
				if ($response['success']) {
					$cryptocurrencies = $response['cryptocurrencies'];
					include __DIR__ . '/includes/cryptocurrencies.php';
				} else {
					wc_clear_notices();
					wc_add_notice('Fsfpay Error: ' . $response['message'], 'error');
					wc_print_notices();
				}
			} else {
				$coin = $_GET['coin'];
				$response = $this->fsfpay_request('payment', [
					'orderId' => $order_id,
					'email' => $order->get_billing_email(),
					'amount' => floatval(number_format(floatval($order_data['total']), 2, '.', '')),
					'currency' => strtoupper($currency),
					'cryptocurrency' => strtoupper($coin),
					'testmode' => $this->test_mode == 'no' ? 0 : 1,
					'callbackUrl' => $this->callback_url,
					'successRedirectUrl' => $order->get_checkout_order_received_url()
				]);
				if ($response['success']) {
					echo '<script type="text/javascript" src="' . $response['scriptUrl'] . '"></script>';
				} else {
					wc_clear_notices();
					wc_add_notice('Fsfpay Error: ' . $response['message'], 'error');
					wc_print_notices();
				}
				global $wp;
				$current_url = home_url($wp->request);
				unset($_GET['coin']);
				echo '<a style="display:inline-block!important;color:#ffffff!important;text-decoration:none!important;margin-top:1rem!important;border:solid 2px #3057d5!important;background:#3057d5!important;box-shadow:0 2px 4px 0 rgba(219,215,215,.5)!important;border-radius:.5rem!important;outline:0!important;padding:.5rem 1rem!important" href="'.htmlspecialchars($current_url.'?'.http_build_query($_GET)).'" class="coin-button">'.__('Pay with Other Cryptocurrency', 'fsfpay-payment').'</a>';
			}
		}
		public function process_payment($order_id)
		{
			$order = wc_get_order($order_id);
			return array(
				'result' => 'success',
				'redirect' => $order->get_checkout_payment_url(true),
			);
		}
		public function payment_scripts()
		{
			if (!is_checkout()) {
				return;
			}
			if ('no' === $this->enabled) {
				return;
			}
		}
		public function callback_handler()
		{
			if (isset($_POST['requestHash']) && $_POST['paymentStatus'] && $_POST['paymentStatus'] == 'successful') {
				if (hash('sha256', $this->api_key . $this->api_username . $this->secret_key) === $_POST['requestHash']) {
					$paymentId = $_POST['paymentId'];
					$orderId = $_POST['orderId'];
					$cryptocurrency = $_POST['cryptocurrency'];
					$cryptocurrencyAmount = $_POST['cryptocurrencyAmount'];
					$testmode = $_POST['testmode'];
					$order = new WC_Order(intval($orderId));
					$order_data = $order->get_data();
					$order_status = $order_data['status'];
					if ($order_status == 'pending' || $order_status == 'failed') {
						if (defined('WOOCOMMERCE_VERSION') && version_compare(WOOCOMMERCE_VERSION, '3.0.0', '<')) {
							$order->reduce_order_stock();
						} else {
							wc_reduce_stock_levels($orderId);
						}
						$note = sprintf(__('Fsfpay Payment Id: %s<br>Payment Amount: %s<br>Payment Mode: %s', 'fsfpay-payment'), '#'.$paymentId, $cryptocurrencyAmount.' '.$cryptocurrency, $testmode == 1 ? __('Test Mode', 'fsfpay-payment') : __('Live Mode', 'fsfpay-payment'));
						$order->add_order_note($note);
						$order->update_status($this->order_status);
					}
					die('OK');
				}
				die('Invalid requestHash!');
			}
			die('Missing fields!');
		}
	}
}
